/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * This file contains a sample program that demonstrates how to enumerate
 * a list of available printers (print queues) from the print service.
 *
 * It's important to note that the example only requests the "printer-name"
 * and "printer-uri-supported" from the print service in this example.  This
 * was done with the papi "dynamic" library in mind.  The papi "dynamic"
 * library is a meta implementation of the PAPI that queries a name service
 * before it bind to a particular print service for a print queue.  When
 * enumerating print queues or querying a particular queue, it attempts to
 * satisfy the request out of the name service data before contacting the
 * print service for additional data.  Since the name service must have a
 * "printer-name" and "printer-uri-supported" for each of it's entries, this
 * will be resolved out of the name service and be relatively quick.  If
 * additional attributes were requested, the print service for each queue
 * might have been contacted for the additional data, which could cause
 * considerable delay in returning the results.
 *
 * It is recommended that applications generating a list of printers limit
 * their initial print queue enumeration to "printer-name" and
 * "printer-uri-supported" in the papiPrintersList() call and additional
 * information be retreived as needed with subsequent papiPrinterQuery() calls.
 * This will improve * application response time in print queue enumeration.
 * Also the use of filters in the papiPrintersList() call is likely to
 * introduce significant delay and overhead.
 */

#pragma ident	"$Id: printers-list.c,v 1.4 2006/01/12 01:31:07 njacobs Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>
#include <libintl.h>
#include <papi.h>

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout,
		gettext("Usage: %s [-E] [-s service]\n"), name);
	exit(1);
}


int
main(int ac, char *av[])
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	papi_printer_t printer = NULL;
	papi_printer_t *printers = NULL;
	char *interest_req[] = { "member-names", NULL };
	char *requested[] = { "printer-name", "printer-uri-supported", NULL };
	char *service = NULL;
	void *iter = NULL;
	int c;

	(void) setlocale(LC_ALL, "");
	(void) textdomain("SUNW_OST_OSCMD");

	while ((c = getopt(ac, av, "s:")) != EOF)
		switch (c) {
		case 'E':	/* type */
			encryption = PAPI_ENCRYPT_ALWAYS;
			break;
		case 's':	/* type */
			service = optarg;
			break;
		default:
			usage(av[0]);
		}

	/* create a connection to the print service */
	status = papiServiceCreate(&svc, service, NULL, NULL, NULL,
					encryption, NULL);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext(
			"Failed to contact print service %s: %s\n"),
			(service ? service : "(default)"),
			papiStatusString(status));
		exit(1);
	}

	/* enumerate the "interest" list of interesting printers. */
	status = papiPrinterQuery(svc, "_all", interest_req, NULL,  &printer);
	if (status == PAPI_OK) {
		papi_attribute_t **attributes = 
				papiPrinterGetAttributeList(printer);	
		char *name = NULL;
		
		for (status = papiAttributeListGetString(attributes, &iter,
						"member-names", &name);
		     status == PAPI_OK;
		     status = papiAttributeListGetString(attributes, &iter,
						NULL, &name)) {
			papi_printer_t p = NULL;

			status = papiPrinterQuery(svc, name, requested, NULL,
						&p);
			if (status == PAPI_OK)
				list_append(&printers, p);
		}
		papiPrinterFree(printer);
	} else {
		/* enumerate a list of all available print queues */
		status = papiPrintersList(svc, requested, NULL, &printers);
		if (status != PAPI_OK) {
			fprintf(stderr, gettext("papiPrintersList(): %s\n"),
				papiStatusString(status));
			exit(1);
		}
	}

	if (printers != NULL) {
		int i;

		/* print the name/uri pair for each queue returned */
		for (i = 0; printers[i] != NULL; i++) {
			papi_attribute_t **attributes =
				papiPrinterGetAttributeList(printers[i]);
			char *name = NULL, *uri = NULL;

			papiAttributeListGetString(attributes, NULL,
					"printer-name", &name);
			papiAttributeListGetString(attributes, NULL,
					"printer-uri-supported", &uri);

			/*
		 	* name should never be NULL, uri may be null if the
		 	* results came from the name service and it was a
		 	* an entry that wasn't specifically for a printer.
		 	* Solaris uses "_default" to specify a default
		 	* destination for various commands and "_all" to
		 	* specify an interest list for commands like
		 	* lpstat and cancel that can operate across "all"
		 	* configured print queues.
		 	*/
			if (uri != NULL)
				printf("%s is %s\n", name, uri);
		}
	}
	papiPrinterListFree(printers);
	papiServiceDestroy(svc);

	exit(0);
}
